/***********************************************************
*                      K O U L E S                         *
*----------------------------------------------------------*
*  C1995 JAHUSOFT                                          *
*        Jan Hubicka                                       *
*        Dukelskych Bojovniku 1944                         *
*        390 03 Tabor                                      *
*        Czech Republic                                    *
*        Phone: 0041-361-32613                             *
*        eMail: hubicka@paru.cas.cz                        *
*----------------------------------------------------------*
*   Copyright(c)1995,1996 by Jan Hubicka.See README for    *
*                     Liecnece details.                    *
*----------------------------------------------------------*
*  cmap.c cmap routines for 256color librarys-svgalib style*
***********************************************************/

#include "koules.h"
static INLINE int
col (int p, CONST float p1)
{
  p *= p1;
  if (p > 63)
    return (63);
  if (p < 0)
    return (0);
  return (p);
}

int             fadedout = 0;
void
setcustompalette (CONST int p, CONST float p1)
{
  /* 0-31 black to yellow for starwars scroller */
  /* 32-63    black to red */
  /* 64-96    for red koules */
  /* 96-128   for yellow rockets */
  /* 128-160   for green rockets */
  /* 160-192   for blue rockets */
  /* 192-256   gray cmap for stars */
  PALETTE         pal;
  int             i;
  for (i = 0; i < 64; i++)
    {
      int             r, g, b;
      r = g = b = 0;
      if ((i & 32) > 0)
        b = (i & 31) << 1;
      if (i < 32)
        {
          r = (i & 3) << 4;     /* 2 bits */
          g = (i & 4) << 3;     /* 1 bit */
          b = (i & 24) << 1;    /* 2 bits */
        }
      pal[i].r = col (r + p, p1);
      pal[i].g = col (g + p, p1);
      pal[i].b = col (b + p, p1);
    }
  for (i = 64; i < 64 + 32; i++)
    {
      int             r, g, b;
      r = g = b = 0;
      r = (32 - (i - 63)) << 1;
      if (i < 64 + 8)
        {
          b = g = (((8 - (i - 63))) << 5) / 5;
        }
      pal[i].r = col (r + p, p1);
      pal[i].g = col (g + p, p1);
      pal[i].b = col (b + p, p1);
    }
  for (i = 96; i < 96 + 32; i++)
    {
      int             r, g, b;
      r = g = b = 0;
      r = g = (32 - (i - 95)) << 1;
      if (i < 96 + 8)
        {
          b = ((8 - (i - 95))) << 3;
        }
      pal[i].r = col (r + p, p1);
      pal[i].g = col (g + p, p1);
      pal[i].b = col (b + p, p1);
    }
  for (i = 128; i < 128 + 32; i++)
    {
      int             r, g, b;
      r = g = b = 0;
      g = (32 - (i - 127)) << 1;
      if (i < 128 + 8)
        {
          r = b = ((8 - (i - 127))) << 3;
        }
      pal[i].r = col (r + p, p1);
      pal[i].g = col (g + p, p1);
      pal[i].b = col (b + p, p1);
    }
  for (i = 160; i < 160 + 32; i++)
    {
      int             r, g, b;
      r = g = b = 0;
      b = (32 - (i - 159)) << 1;
      if (i < 160 + 8)
        {
          r = g = (((8 - (i - 159))) << 3) / 2;
        }
      pal[i].r = col (r + p, p1);
      pal[i].g = col (g + p, p1);
      pal[i].b = col (b + p, p1);
    }
  for (i = 0; i < 32; i++)
    {
      pal[i].r = col (i * 2 + p, p1);
      pal[i].g = col (i * 2 + p, p1);
      pal[i].b = col (p, p1);
    }
  for (i = 0; i < 32; i++)
    {
      pal[192 + i].r = col (i * 2 + p, p1);
      pal[192 + i].g = col (i * 2 + p, p1);
      pal[192 + i].b = col (i * 2 + p, p1);
    }
  pal[0].r = 0;
  pal[0].g = 0;
  pal[0].b = 0;
  pal[255].r = col (64 + p, p1);
  pal[255].g = col (64 + p, p1);
  pal[255].b = col (64 + p, p1);
  vsync ();
  set_palette (pal);
}
void
fadeout ()
{
  if (!fadedout)
    {
      float           i;
      for (i = 1; i >= 0; i -= 0.1)
        {
          setcustompalette (0, i);
          usleep (200), tbreak = 1;

        }
      setcustompalette (-65, 0);
      fadedout = 1;
    }
}
void
fadein ()
{
  if (fadedout)
    {
      float           i;
      for (i = 0; i <= 1; i += 0.1)
        {
          setcustompalette (0, i);
          usleep (200), tbreak = 1;

        }
      setcustompalette (0, 1);
      fadedout = 0;
    }
}
void
fadein1 ()                      /*better for star background */
{
  if (fadedout)
    {
      int             i;
      for (i = -64; i <= 0; i += 6)
        {
          setcustompalette (i, 1);
          usleep (200), tbreak = 1;

        }
      setcustompalette (0, 1);
      fadedout = 0;
    }
}
