#include <stdio.h>
#ifdef JOYSTICK
#include <sys/ioctl.h>
#include "joystick.h"
#include "koules.h"
#include "joy.h"
int joystickdevice[2] =
{-1, -1};
int calibrated[2];
int center[2][2];
char *calibratemessage;
static int oldapi=1;
static int positions[2][2];
static int buttons[2];
static int joypos;

static struct JS_DATA_TYPE js_data;
int
joystick_init ()
{
  int version;
  joystickdevice[0] = open ("/dev/js0", O_RDONLY);
  if (joystickdevice[0] < 0)
    {
      perror ("Joystick driver");
      printf ("Joystick 1 not avaiable..\n");
      joystickplayer[0] = -1;
      joystickplayer[1] = -1;
      return 0;
    }
  else
    printf ("Joystick 1 found\n");
  if (ioctl (joystickdevice[0], JSIOCGVERSION, &version) == -1)
    {
      printf ("Using old joystick API\n");
    }
  else
    {
      printf ("Joystick driver using new API\n"), oldapi = 0;
      close (joystickdevice[0]);
      joystickdevice[0] = open ("/dev/js0", O_RDONLY | O_NONBLOCK);
      if (joystickdevice[0] < 0)
	{
	  printf ("Nonblocking open failed. Aborting joystick initialization\n");
	  joystickplayer[0] = -1;
	  joystickplayer[1] = -1;
	  return 0;
	}
      calibrated[0] = 1;
    }
  joystickdevice[1] = open ("/dev/js1", O_RDONLY | (!oldapi ? O_NONBLOCK : 0));
  if (joystickdevice[1] < 0)
    {
      perror ("Joystick driver");
      printf ("Joystick 2 not avaiable..\n");
      joystickplayer[1] = -1;
    }
  else
    {
      printf ("Joystick 2 initialized\n");
      if (!oldapi)
	calibrated[0] = 1;
    }
  return 1;
}

/* Fill joystickplayer. Coordinates are in the range -1, 1 */
void
process_joystick ()
{
  int status;
  struct JS_DATA_TYPE js;
  int i;
  if (!oldapi)
    {
      struct js_event e;
      for (i = 0; i < 2; i++)
	{
	  while (read (joystickdevice[i], &e, sizeof (struct js_event)) > 0)
	    {
	      switch (e.type & ~JS_EVENT_INIT)
		{
		case JS_EVENT_BUTTON:
		  buttons[i] = e.value;
		  break;
		case JS_EVENT_AXIS:
		  if (e.number < 2)
		    positions[i][e.number] = e.value;
		}
	    }
	}
      for (i = 0; i < 2; i++)
	{
	  if (joystickplayer[i] >= 0)
	    {
	      if (object[joystickplayer[i]].type != ROCKET)
		continue;
	      controls[joystickplayer[i]].jx = positions[i][0] / 32767.0;
	      controls[joystickplayer[i]].jy = positions[i][1] / 32767.0;
	      controls[joystickplayer[i]].mask = buttons[i];
	      controls[joystickplayer[i]].type = C_JOYSTICK1;
	    }
	}

      return;
    }
  for (i = 0; i < 2; i++)
    {
      double x, y;
      if (joystickplayer[i] >= 0)
	{
	  if (object[joystickplayer[i]].type != ROCKET)
	    continue;
	  status = read (joystickdevice[i], &js, JS_RETURN);
	  if (status != JS_RETURN)
	    {
	      break;
	    }
	  x = center[i][0] - js.x;
	  y = center[i][1] - js.y;
	  controls[joystickplayer[i]].jx = -x / 128.0;
	  controls[joystickplayer[i]].jy = -y / 128.0;
	  controls[joystickplayer[i]].mask = js.buttons;
	  controls[joystickplayer[i]].type = C_JOYSTICK1;
	}
    }
}

/*Return values:
   -1 failure - disable joystick forever

   joypos is 0 for "move to lower right corner
   1 for "center joystick"
 */
int
joystick_calibrate (int joystick, int *x, int *y)
{
  int status, tmp;
  static int lastbutton, button;
  if (!oldapi) {
	  if (joystickdevice[joystick]>=0) {calibrated[joystick]=1;return 1;}
	  return -1;}
  if(joypos>1) return 0;
  if (read (joystickdevice[joystick], &js_data, JS_RETURN) <= 0)
    {
      perror ("Joystick");
      return -1;
    }
  *x = js_data.x;
  *y = js_data.y;
  button = lastbutton;
  lastbutton = js_data.buttons + IsPressedEnter ();
  if (!joypos)
    {
      if ((button == 0x00 || lastbutton != 0x00))
	return 1;

      for (tmp = 0; js_data.x > 0xff; tmp++, js_data.x = js_data.x >> 1);
      js_data.x = tmp;
      for (tmp = 0; js_data.y > 0xff; tmp++, js_data.y = js_data.y >> 1);
      js_data.y = tmp;


      status = ioctl (joystickdevice[joystick], JS_SET_CAL, &js_data);
      if (status == -1)
	{
	  perror ("Joystick");

	}
      calibratemessage="Move stick to center";
      joypos++;
      return 1;
    }
  if ((button == 0x00 || lastbutton != 0x00))
    return 1;

  calibrated[joystick] = 1;
  center[joystick][0] = js_data.x;
  center[joystick][1] = js_data.y;
  return 0;
}

/*return values:
   0 - joystick not available
   1 - OK
   2 - calibration is not neccesary (and available)
 */
int
calibrate_init (int joystick)
{
  int status;
  long tmpl;
   joypos=0;
      calibratemessage="Move stick to bottom right corner";
  if (joystickdevice[joystick] < 0)
    {
      printf ("Joystick %c not found during initialization.\n", joystick + 'A');
      return 0;
    }
  if (!oldapi) 
  {
	  return 2;
	  calibrated[joystick]=1;
  }

  status = ioctl (joystickdevice[joystick], JS_GET_TIMELIMIT, &tmpl);
  if (status == -1)
    {
      perror ("Joystick");
      return 0;
    }

  tmpl = 10;			/* 10miliseconds */
  status = ioctl (joystickdevice[joystick], JS_SET_TIMELIMIT, &tmpl);
  if (status == -1)
    {
      perror ("Joystick");
      return 0;
    }
  return 1;
}
#endif
